/* www.sgm.siedlce.pl

Prosta rozladowarka 'SGM ArduSto' do pakietow
LiPo (2S-3S), realizujaca funkcje "storage"

Dioda (LED13) sygnalizuje po podlaczeniu rozpoznana ilosc cel,
Prog rozladowania przelaczany zworka/przelacznikiem,
dwie wartosci progu rozladowania ustawiane w wierszu deklaracji:
"const float UminH = 3.7;"
"const float UminL = 3.6;"

Zaleca sie wstawienie do programu fizycznych wartosci
ponizszych parametrów, zmierzonych miernikiem uniwersalnym
"const float Ref = 5;"  --> napiecie wyjsciowe stabilizatora LM2940 
"const float Uf = 0.75;" --> napiecie przewodzenia Uf diody D1
"const float x1 = 2.771;" -->  dzielnik pomiarowy R4/R5 

Projekt wylacznie do wykorzystania na wlasny uzytek
*/


//********* rzeczywiste wartosci do ustawienia w konkretnym ukladzie 
const float UminL = 3.68;   //napiecie min(Low) na 1 cele (prog rozladowania)
const float UminH = 3.76;   //napiecie min(High)
const float Ref = 5.02;     //napiecie odniesienia (zmierzone, stabilizator 5V)
const float Uf = 0.77;      //napiecie przewodzenia Uf diody D1 (zmierzone)
const float x1 = 2.771;     //dzielnik pomiarowy - wstawic zmierzona i obliczona wartosc
                            //u mnie 3k9/6k8 --> teoretycznie 2,743, zmierzone 2,771
//*********


const int LED = 13;         // LED na Pin 13
const int BUZ = 12;         // wyjscie Buzzer na Pin 12 
const int REL = 11;         // wyjscie Relay na Pin 11
byte DisOK = 0;             // rozladowanie rozpoczete
byte nCell;                 // ilosc cel
int ReadA0;                 // odczyt we analog0
int TstZW;                  //odczyt zworki/SW
float Umin;                 //zmienna na prog rozldowania
float Up;                   //obliczone napiecie pakietu 
const float Umax = 4.3;     //napiecie max na 1 cele
const byte Cmax = 3;        //maksymalna ilosc cel, mozna zwiekszyc z uwzglednieniem
                            //zastrzezen wymienionych w artykule


//zmienne do wydrukow testowych
unsigned long LastTime = 0; //zmienna do drukowania co 'PrintGap'
int PrintGap = 5000;        //co ile wydruk (msec)
int nPrint = 1;             //numer kolejny wydruku


void setup()
{
  //Serial.begin(9600);    // do testowania

  //czytaj stan zwory napiecia rozladowania, wejscie A2
  TstZW = analogRead(2);
  delay(5);
  if (TstZW <500){
   Umin =  UminH;  //zworka zdjeta
  } else {
   Umin =  UminL;  //zworka zalozona
  }
  
  pinMode(LED, OUTPUT);
  digitalWrite(LED, HIGH);  //LED wlaczony, sygnalizacja podlaczenia pakietu

  pinMode(REL, OUTPUT);
  digitalWrite(REL, LOW);  //przekaznik wylaczony

  pinMode(BUZ, OUTPUT);    //potwierdzenie inicjalizacji krotkim 'beep'
  digitalWrite(BUZ, HIGH);
  delay (300);
  digitalWrite(BUZ, LOW);

  if (TstZW > 499){       //drugie pikniecie jesli zworka zalozona
   delay (200); 
   digitalWrite(BUZ, HIGH);
   delay (300);
   digitalWrite(BUZ, LOW);
  }
 

}


void loop() {

  TstZW = analogRead(2);  //czytaj stan zwory napiecia rozladowania, wejscie A2
  delay(5);
  if (TstZW <500){
   Umin =  UminH;  //zworka zdjeta
  } else {
   Umin =  UminL;  //zworka zalozona
  }
  delay(5);
  
  ReadA0 = analogRead(0);               //pomiar napiecia na A0 
  delay(5);

  Up = Uf + (x1 * Ref * ReadA0) / 1024; //obliczenie zmierzonego napiecia
  nCell = 0;

/*
  if ((Up > 2 * Umin) && (Up < 2 * Umax)) {
    nCell = 2;
  }
  if ((Up > 3 * Umin) && (Up < 3 * Umax)) {
    nCell = 3;
  }
*/  
  // to samo w petli
  for (byte i = 2; i < Cmax+1 ; i++) {
   if ((Up > i * Umin) && (Up < i * Umax)) {
    nCell = i;
   }
  }

  //wydruk zmierzonych wartości napięcia podczas testowania
  if (millis() - LastTime > PrintGap) {
    LastTime = millis(); 
    //Serial.print("n: ");
    //Serial.print(nPrint);  //numer kolejny
    //Serial.print("; C:");
    //Serial.print(nCell);   //rozpoznana ilosc cel
    //Serial.print(";  V:";
    //Serial.println(Up,3);  //napiecie pakietu
    nPrint++;
  }

 if (nCell > 0) {
  digitalWrite(REL, HIGH);
  DisOK = 1;               //rozladowanie rozpoczete
 } else {
   if (DisOK =1) {
    for (byte i = 0; i < 10; i++) { //sygnalizacja dzwiekowa zakonczenia rozladowania
     digitalWrite(BUZ, HIGH);       
     delay(500);
     digitalWrite(BUZ, LOW);
     delay(500);    
    }
   }
   digitalWrite(REL, LOW);  // dopracować
   delay(1000); //5000
 }

 for (byte i = 0; i < nCell; i++) {   //ilosc blyskow LED = ilosc cel LiPo
    digitalWrite(LED, HIGH);
    delay(100);
    digitalWrite(LED, LOW);
    delay(200);
 }
 delay (1000);

} //void loop()


